/**
  * elVueCompositionUtils v0.1.0
  * (c) 2021 Abdulnasır olcan
  * @license MIT
  */
import { ref, readonly, computed, watch, reactive, onMounted, onUnmounted, toRefs, watchEffect } from '@vue/composition-api';
import Vue from 'vue';

/**
 * This function is used in reactive state.
 * @param {initial} - The initial argument is the state used during the initial render.
 * @returns - state and set
 */

function useState(initial) {
  const state = ref(initial);
  const set = val => {
    state.value = val;
  };
  return [readonly(state), set];
}

/**
 * This function update value when another value changes (data-binding).
 * @param {initial} - The initial is the state used during the initial render.
 * @returns  set, sort, filter, first, last, list, deleteFirst, deleteLast, reduce, reset, push,
 */

const useList = (initialList = []) => {
  const [list, setList] = useState(initialList);

  const isList = list => list || [];
  const isArray = list => (Array.isArray(list) ? [] : {});

  const push = (...values) => {
    const pushed = list => isList([...list, ...values]);
    return setList(pushed(list.value));
  };

  const sort = fn => {
    const sorted = (funs, list) => [...isList(list)].sort(funs);
    return setList(sorted(fn, list.value));
  };

  const filter = fn => {
    const filtered = (funs, list) => [...isList(list)].filter(funs);
    return setList(filtered(fn, list.value));
  };

  const reduce = fn => {
    if (!Array.isArray(list.value)) return;
    const reduced = (funs, list) => [...isList(list)].reduce(funs, isArray(list));
    return setList(reduced(fn, list.value));
  };

  const first = () => {
    const [first] = isList(list.value);
    return setList(first);
  };

  const last = () => {
    const { length, [length - 1]: last } = isList(list.value);
    return setList(last);
  };

  const deleteFirst = () => {
    const [, ...rest] = isList(list.value);
    return setList(rest);
  };

  const deleteLast = () => {
    const { length, [length - 1]: last } = list.value;
    const filtered = list.value.filter(v => v !== last);
    return setList(filtered);
  };

  const reset = () => setList(initialList);

  return {
    set: setList,
    sort,
    filter,
    first,
    last,
    list,
    deleteFirst,
    deleteLast,
    reduce,
    reset,
    push,
  };
};

const isArray = Array.isArray;

const isObject = val => val !== null && typeof val === 'object';

/**
 * This function provides embed code related controls
 * @param {object} code This object contain two parameter.
 * @returns {object} object
 * @example{
    usePick(
      {
        a: 1,
        b: 2,
        c: 3,
        d: 4,
      },
      ['a', 'd'],
    )

   exit: {a: 1, d: 4}
 */

const usePick = (obj, arr = []) => {
  if (isArray(obj) || !isObject(obj)) return obj;

  let keys = isArray(arr) ? arr : Object.keys(arr);
  const mapping = !isArray(arr);

  return keys.reduce((picked, key) => {
    const newKey = mapping ? arr[key] : key;
    picked[newKey] = obj[key];

    return picked;
  }, {});
};

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var dayjs_min = createCommonjsModule(function (module, exports) {
!function(t,e){module.exports=e();}(commonjsGlobal,function(){var t="millisecond",e="second",n="minute",r="hour",i="day",s="week",u="month",a="quarter",o="year",f="date",h=/^(\d{4})[-/]?(\d{1,2})?[-/]?(\d{0,2})[^0-9]*(\d{1,2})?:?(\d{1,2})?:?(\d{1,2})?[.:]?(\d+)?$/,c=/\[([^\]]+)]|Y{1,4}|M{1,4}|D{1,2}|d{1,4}|H{1,2}|h{1,2}|a|A|m{1,2}|s{1,2}|Z{1,2}|SSS/g,d={name:"en",weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_")},$=function(t,e,n){var r=String(t);return !r||r.length>=e?t:""+Array(e+1-r.length).join(n)+t},l={s:$,z:function(t){var e=-t.utcOffset(),n=Math.abs(e),r=Math.floor(n/60),i=n%60;return (e<=0?"+":"-")+$(r,2,"0")+":"+$(i,2,"0")},m:function t(e,n){if(e.date()<n.date())return -t(n,e);var r=12*(n.year()-e.year())+(n.month()-e.month()),i=e.clone().add(r,u),s=n-i<0,a=e.clone().add(r+(s?-1:1),u);return +(-(r+(n-i)/(s?i-a:a-i))||0)},a:function(t){return t<0?Math.ceil(t)||0:Math.floor(t)},p:function(h){return {M:u,y:o,w:s,d:i,D:f,h:r,m:n,s:e,ms:t,Q:a}[h]||String(h||"").toLowerCase().replace(/s$/,"")},u:function(t){return void 0===t}},y="en",M={};M[y]=d;var m=function(t){return t instanceof S},D=function(t,e,n){var r;if(!t)return y;if("string"==typeof t)M[t]&&(r=t),e&&(M[t]=e,r=t);else {var i=t.name;M[i]=t,r=i;}return !n&&r&&(y=r),r||!n&&y},v=function(t,e){if(m(t))return t.clone();var n="object"==typeof e?e:{};return n.date=t,n.args=arguments,new S(n)},g=l;g.l=D,g.i=m,g.w=function(t,e){return v(t,{locale:e.$L,utc:e.$u,x:e.$x,$offset:e.$offset})};var S=function(){function d(t){this.$L=D(t.locale,null,!0),this.parse(t);}var $=d.prototype;return $.parse=function(t){this.$d=function(t){var e=t.date,n=t.utc;if(null===e)return new Date(NaN);if(g.u(e))return new Date;if(e instanceof Date)return new Date(e);if("string"==typeof e&&!/Z$/i.test(e)){var r=e.match(h);if(r){var i=r[2]-1||0,s=(r[7]||"0").substring(0,3);return n?new Date(Date.UTC(r[1],i,r[3]||1,r[4]||0,r[5]||0,r[6]||0,s)):new Date(r[1],i,r[3]||1,r[4]||0,r[5]||0,r[6]||0,s)}}return new Date(e)}(t),this.$x=t.x||{},this.init();},$.init=function(){var t=this.$d;this.$y=t.getFullYear(),this.$M=t.getMonth(),this.$D=t.getDate(),this.$W=t.getDay(),this.$H=t.getHours(),this.$m=t.getMinutes(),this.$s=t.getSeconds(),this.$ms=t.getMilliseconds();},$.$utils=function(){return g},$.isValid=function(){return !("Invalid Date"===this.$d.toString())},$.isSame=function(t,e){var n=v(t);return this.startOf(e)<=n&&n<=this.endOf(e)},$.isAfter=function(t,e){return v(t)<this.startOf(e)},$.isBefore=function(t,e){return this.endOf(e)<v(t)},$.$g=function(t,e,n){return g.u(t)?this[e]:this.set(n,t)},$.unix=function(){return Math.floor(this.valueOf()/1e3)},$.valueOf=function(){return this.$d.getTime()},$.startOf=function(t,a){var h=this,c=!!g.u(a)||a,d=g.p(t),$=function(t,e){var n=g.w(h.$u?Date.UTC(h.$y,e,t):new Date(h.$y,e,t),h);return c?n:n.endOf(i)},l=function(t,e){return g.w(h.toDate()[t].apply(h.toDate("s"),(c?[0,0,0,0]:[23,59,59,999]).slice(e)),h)},y=this.$W,M=this.$M,m=this.$D,D="set"+(this.$u?"UTC":"");switch(d){case o:return c?$(1,0):$(31,11);case u:return c?$(1,M):$(0,M+1);case s:var v=this.$locale().weekStart||0,S=(y<v?y+7:y)-v;return $(c?m-S:m+(6-S),M);case i:case f:return l(D+"Hours",0);case r:return l(D+"Minutes",1);case n:return l(D+"Seconds",2);case e:return l(D+"Milliseconds",3);default:return this.clone()}},$.endOf=function(t){return this.startOf(t,!1)},$.$set=function(s,a){var h,c=g.p(s),d="set"+(this.$u?"UTC":""),$=(h={},h[i]=d+"Date",h[f]=d+"Date",h[u]=d+"Month",h[o]=d+"FullYear",h[r]=d+"Hours",h[n]=d+"Minutes",h[e]=d+"Seconds",h[t]=d+"Milliseconds",h)[c],l=c===i?this.$D+(a-this.$W):a;if(c===u||c===o){var y=this.clone().set(f,1);y.$d[$](l),y.init(),this.$d=y.set(f,Math.min(this.$D,y.daysInMonth())).$d;}else $&&this.$d[$](l);return this.init(),this},$.set=function(t,e){return this.clone().$set(t,e)},$.get=function(t){return this[g.p(t)]()},$.add=function(t,a){var f,h=this;t=Number(t);var c=g.p(a),d=function(e){var n=v(h);return g.w(n.date(n.date()+Math.round(e*t)),h)};if(c===u)return this.set(u,this.$M+t);if(c===o)return this.set(o,this.$y+t);if(c===i)return d(1);if(c===s)return d(7);var $=(f={},f[n]=6e4,f[r]=36e5,f[e]=1e3,f)[c]||1,l=this.$d.getTime()+t*$;return g.w(l,this)},$.subtract=function(t,e){return this.add(-1*t,e)},$.format=function(t){var e=this;if(!this.isValid())return "Invalid Date";var n=t||"YYYY-MM-DDTHH:mm:ssZ",r=g.z(this),i=this.$locale(),s=this.$H,u=this.$m,a=this.$M,o=i.weekdays,f=i.months,h=function(t,r,i,s){return t&&(t[r]||t(e,n))||i[r].substr(0,s)},d=function(t){return g.s(s%12||12,t,"0")},$=i.meridiem||function(t,e,n){var r=t<12?"AM":"PM";return n?r.toLowerCase():r},l={YY:String(this.$y).slice(-2),YYYY:this.$y,M:a+1,MM:g.s(a+1,2,"0"),MMM:h(i.monthsShort,a,f,3),MMMM:h(f,a),D:this.$D,DD:g.s(this.$D,2,"0"),d:String(this.$W),dd:h(i.weekdaysMin,this.$W,o,2),ddd:h(i.weekdaysShort,this.$W,o,3),dddd:o[this.$W],H:String(s),HH:g.s(s,2,"0"),h:d(1),hh:d(2),a:$(s,u,!0),A:$(s,u,!1),m:String(u),mm:g.s(u,2,"0"),s:String(this.$s),ss:g.s(this.$s,2,"0"),SSS:g.s(this.$ms,3,"0"),Z:r};return n.replace(c,function(t,e){return e||l[t]||r.replace(":","")})},$.utcOffset=function(){return 15*-Math.round(this.$d.getTimezoneOffset()/15)},$.diff=function(t,f,h){var c,d=g.p(f),$=v(t),l=6e4*($.utcOffset()-this.utcOffset()),y=this-$,M=g.m(this,$);return M=(c={},c[o]=M/12,c[u]=M,c[a]=M/3,c[s]=(y-l)/6048e5,c[i]=(y-l)/864e5,c[r]=y/36e5,c[n]=y/6e4,c[e]=y/1e3,c)[d]||y,h?M:g.a(M)},$.daysInMonth=function(){return this.endOf(u).$D},$.$locale=function(){return M[this.$L]},$.locale=function(t,e){if(!t)return this.$L;var n=this.clone(),r=D(t,e,!0);return r&&(n.$L=r),n},$.clone=function(){return g.w(this.$d,this)},$.toDate=function(){return new Date(this.valueOf())},$.toJSON=function(){return this.isValid()?this.toISOString():null},$.toISOString=function(){return this.$d.toISOString()},$.toString=function(){return this.$d.toUTCString()},d}(),p=S.prototype;return v.prototype=p,[["$ms",t],["$s",e],["$m",n],["$H",r],["$W",i],["$M",u],["$y",o],["$D",f]].forEach(function(t){p[t[1]]=function(e){return this.$g(e,t[0],t[1])};}),v.extend=function(t,e){return t.$i||(t(e,S,v),t.$i=!0),v},v.locale=D,v.isDayjs=m,v.unix=function(t){return v(1e3*t)},v.en=M[y],v.Ls=M,v.p={},v});
});

var relativeTime = createCommonjsModule(function (module, exports) {
!function(r,t){module.exports=t();}(commonjsGlobal,function(){return function(r,t,e){r=r||{};var n=t.prototype,o={future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"};function i(r,t,e,o){return n.fromToBase(r,t,e,o)}e.en.relativeTime=o,n.fromToBase=function(t,n,i,d,u){for(var a,f,s,l=i.$locale().relativeTime||o,h=r.thresholds||[{l:"s",r:44,d:"second"},{l:"m",r:89},{l:"mm",r:44,d:"minute"},{l:"h",r:89},{l:"hh",r:21,d:"hour"},{l:"d",r:35},{l:"dd",r:25,d:"day"},{l:"M",r:45},{l:"MM",r:10,d:"month"},{l:"y",r:17},{l:"yy",d:"year"}],m=h.length,c=0;c<m;c+=1){var y=h[c];y.d&&(a=d?e(t).diff(i,y.d,!0):i.diff(t,y.d,!0));var p=(r.rounding||Math.round)(Math.abs(a));if(s=a>0,p<=y.r||!y.r){p<=1&&c>0&&(y=h[c-1]);var v=l[y.l];u&&(p=u(""+p)),f="string"==typeof v?v.replace("%d",p):v(p,n,y.l,s);break}}if(n)return f;var M=s?l.future:l.past;return "function"==typeof M?M(f):M.replace("%s",f)},n.to=function(r,t){return i(r,t,this,!0)},n.from=function(r,t){return i(r,t,this)};var d=function(r){return r.$u?e.utc():e()};n.toNow=function(r){return this.to(d(this),r)},n.fromNow=function(r){return this.from(d(this),r)};}});
});

var utc = createCommonjsModule(function (module, exports) {
!function(t,i){module.exports=i();}(commonjsGlobal,function(){return function(t,i,e){var s=i.prototype;e.utc=function(t){return new i({date:t,utc:!0,args:arguments})},s.utc=function(t){var i=e(this.toDate(),{locale:this.$L,utc:!0});return t?i.add(this.utcOffset(),"minute"):i},s.local=function(){return e(this.toDate(),{locale:this.$L,utc:!1})};var f=s.parse;s.parse=function(t){t.utc&&(this.$u=!0),this.$utils().u(t.$offset)||(this.$offset=t.$offset),f.call(this,t);};var n=s.init;s.init=function(){if(this.$u){var t=this.$d;this.$y=t.getUTCFullYear(),this.$M=t.getUTCMonth(),this.$D=t.getUTCDate(),this.$W=t.getUTCDay(),this.$H=t.getUTCHours(),this.$m=t.getUTCMinutes(),this.$s=t.getUTCSeconds(),this.$ms=t.getUTCMilliseconds();}else n.call(this);};var u=s.utcOffset;s.utcOffset=function(t,i){var e=this.$utils().u;if(e(t))return this.$u?0:e(this.$offset)?u.call(this):this.$offset;var s=Math.abs(t)<=16?60*t:t,f=this;if(i)return f.$offset=s,f.$u=0===t,f;if(0!==t){var n=this.$u?this.toDate().getTimezoneOffset():-1*this.utcOffset();(f=this.local().add(s+n,"minute")).$offset=s,f.$x.$localOffset=n;}else f=this.utc();return f};var o=s.format;s.format=function(t){var i=t||(this.$u?"YYYY-MM-DDTHH:mm:ss[Z]":"");return o.call(this,i)},s.valueOf=function(){var t=this.$utils().u(this.$offset)?0:this.$offset+(this.$x.$localOffset||(new Date).getTimezoneOffset());return this.$d.valueOf()-6e4*t},s.isUTC=function(){return !!this.$u},s.toISOString=function(){return this.toDate().toISOString()},s.toString=function(){return this.toDate().toUTCString()};var r=s.toDate;s.toDate=function(t){return "s"===t&&this.$offset?e(this.format("YYYY-MM-DD HH:mm:ss:SSS")).toDate():r.call(this)};var a=s.diff;s.diff=function(t,i,s){if(t&&this.$u===t.$u)return a.call(this,t,i,s);var f=this.local(),n=e(t).local();return a.call(f,n,i,s)};}});
});

var timezone = createCommonjsModule(function (module, exports) {
!function(t,e){module.exports=e();}(commonjsGlobal,function(){var t={year:0,month:1,day:2,hour:3,minute:4,second:5},e={};return function(n,i,r){var o,u=r().utcOffset(),a=function(t,n,i){void 0===i&&(i={});var r=new Date(t);return function(t,n){void 0===n&&(n={});var i=n.timeZoneName||"short",r=t+"|"+i,o=e[r];return o||(o=new Intl.DateTimeFormat("en-US",{hour12:!1,timeZone:t,year:"numeric",month:"2-digit",day:"2-digit",hour:"2-digit",minute:"2-digit",second:"2-digit",timeZoneName:i}),e[r]=o),o}(n,i).formatToParts(r)},f=function(e,n){for(var i=a(e,n),o=[],u=0;u<i.length;u+=1){var f=i[u],s=f.type,m=f.value,c=t[s];c>=0&&(o[c]=parseInt(m,10));}var d=o[3],v=24===d?0:d,h=o[0]+"-"+o[1]+"-"+o[2]+" "+v+":"+o[4]+":"+o[5]+":000",l=+e;return (r.utc(h).valueOf()-(l-=l%1e3))/6e4},s=i.prototype;s.tz=function(t,e){void 0===t&&(t=o);var n=this.utcOffset(),i=this.toDate().toLocaleString("en-US",{timeZone:t}),a=Math.round((this.toDate()-new Date(i))/1e3/60),f=r(i).$set("millisecond",this.$ms).utcOffset(u-a,!0);if(e){var s=f.utcOffset();f=f.add(n-s,"minute");}return f.$x.$timezone=t,f},s.offsetName=function(t){var e=this.$x.$timezone||r.tz.guess(),n=a(this.valueOf(),e,{timeZoneName:t}).find(function(t){return "timezonename"===t.type.toLowerCase()});return n&&n.value};var m=s.startOf;s.startOf=function(t,e){if(!this.$x||!this.$x.$timezone)return m.call(this,t,e);var n=r(this.format("YYYY-MM-DD HH:mm:ss:SSS"));return m.call(n,t,e).tz(this.$x.$timezone,!0)},r.tz=function(t,e,n){var i=n&&e,u=n||e||o,a=f(+r(),u);if("string"!=typeof t)return r(t).tz(u);var s=function(t,e,n){var i=t-60*e*1e3,r=f(i,n);if(e===r)return [i,e];var o=f(i-=60*(r-e)*1e3,n);return r===o?[i,r]:[t-60*Math.min(r,o)*1e3,Math.max(r,o)]}(r.utc(t,i).valueOf(),a,u),m=s[0],c=s[1],d=r(m).utcOffset(c);return d.$x.$timezone=u,d},r.tz.guess=function(){return Intl.DateTimeFormat().resolvedOptions().timeZone},r.tz.setDefault=function(t){o=t;};}});
});

dayjs_min.extend(relativeTime);
dayjs_min.extend(utc);
dayjs_min.extend(timezone);

const useDate = () => {
  const format = (date, dateFormat) => {
    return dayjs_min(date).format(dateFormat);
  };

  const timeAgo = (date, time) => {
    return dayjs_min(date).to(dayjs_min(time));
  };

  const getDate = unit => {
    return dayjs_min().get(unit);
  };

  const utc = (date, format) => {
    return dayjs_min(date).utc().format(format);
  };

  const timezone = (date, local, format) => {
    return dayjs_min(date).tz(local).format(format);
  };

  const difference = (datePri, dateSec, unit) => {
    const datePrimary = dayjs_min(datePri);
    const dateSecondary = dayjs_min(dateSec);

    return datePrimary.diff(dateSecondary, unit);
  };

  return {
    format,
    timeAgo,
    getDate,
    utc,
    timezone,
    difference,
  };
};

/**
 * This function update value when another value changes (data-binding).
 * @param {initial} - The initial is the state used during the initial render.
 * @returns value, setValue, changed, reset
 */

const useBind = initial => {
  const [value, setValue] = useState(initial);

  const reset = () => setValue(initial);
  const changed = e => setValue(e.target.value);

  return {
    value,
    setValue,
    changed,
    reset,
  };
};

const useQueue = (initial = []) => {
  const [queue, setQueue] = useState(initial);

  const set = value => {
    return setQueue(value);
  };

  const first = computed(() => {
    const [first] = queue.value;
    return first;
  });

  const last = computed(() => {
    const { length, [length - 1]: last } = queue.value;
    return last;
  });

  const size = computed(() => {
    return queue.value.length;
  });

  const remove = () => {
    let result;
    const [first, ...rest] = queue.value;
    const removed = ([first, ...rest]) => {
      result = first;
      return rest;
    };
    setQueue(removed([first, ...rest]));
    return result;
  };

  return {
    state: queue,
    set,
    remove,
    first,
    last,
    size,
  };
};

/**
 * This function provides embed code related controls
 * @param {object} code This object contain two parameter.
 * @returns {object} object
 * @example{
 *
    const code = ref(null);
    const { getEmbedScriptSrc, injectScript, isEmbedBlock, clearScript } = useEmbed(code);

 *  getEmbedScriptSrc: Returns embed code property  // width, height
 *
 *   return{
 *     isEmbedBlock: Provides information whether the value entered is suitable embed code pattern or not   // true | false
 *     clear
 *   }
 * }
 */

const useEmbed = (code = null) => {
  const embedCode = ref(code);
  const injectedScripts = ref([]);

  const isEmbedBlock = computed(
    () =>
      /(?:<iframe[^>]*)(?:(?:\/>)|(?:>.*?<\/iframe>))/.test(embedCode.value) ||
      /(?:<blockquote[^>]*)(?:(?:\/>)|(?:>.*?<\/blockquote>))/.test(embedCode.value),
  );

  const getEmbedScriptSrc = (embedString = embedCode.value) => {
    const parser = new DOMParser();
    const parsedCode = parser.parseFromString(embedString, 'text/html');

    for (const node of parsedCode.body.childNodes) {
      if (node.tagName === 'SCRIPT') {
        return node.src;
      } else if (node.innerHTML && node.innerHTML.includes('<script')) {
        getEmbedScriptSrc(node.innerHTML);
      }
    }

    return null;
  };

  const clearScript = scriptEl => scriptEl.remove();

  const injectScript = ({ async = true, defer = false, id, src }) => {
    const scriptWithSameId = document.getElementById(id);

    if (scriptWithSameId) {
      clearScript(scriptWithSameId);
    }
    const script = document.createElement('script');
    script.type = 'text/javascript';
    script.id = id;
    script.src = src;
    script.async = async;
    script.defer = defer;
    document.body.insertAdjacentElement('afterend', script);

    script.addEventListener('load', () => {
      if (window.instgrm) {
        window.instgrm.Embeds.process();
      }
    });

    injectedScripts.value = [...injectedScripts.value, script];
  };

  const clear = () => {
    embedCode.value = null;
    injectedScripts.value.map(script => clearScript(script));
    injectedScripts.value = [];
  };

  watch(code, newValue => {
    if (newValue) {
      const src = getEmbedScriptSrc(newValue);
      injectScript({ id: 'id', src });
    }
  });

  return {
    isEmbedBlock,
    clear,
  };
};

const useModal = (initItem = {}, initVisible = false) => {
  const [visible, setVisible] = useState(initVisible);

  const [current, setCurrent] = useState(initItem);

  const openModal = item => {
    setCurrent(item);
    setVisible(true);
  };

  const closeModal = () => {
    setVisible(false);
    setCurrent();
  };

  return { visible, setVisible, current, openModal, closeModal };
};

const useResize = (el = null) => {
  let state = reactive({
    screenWidth: 0,
    screenHeight: 0,
    ratiowh: 0,
    ratiohw: 0,
    rect: undefined,
  });

  // eslint-disable-next-line no-unused-vars
  const onResize = event => {
    state.screenWidth = window.innerWidth;
    state.screenHeight = window.innerHeight;
    state.ratiowh = state.screenWidth / state.screenHeight;
    state.ratiohw = state.screenHeight / state.screenWidth;

    if (el && el.value) {
      const clientRect = el.value.getBoundingClientRect();
      state.rect = {
        width: clientRect.width,
        height: clientRect.height,
        left: clientRect.left,
        right: clientRect.right,
        top: clientRect.top,
        bottom: clientRect.bottom,
      };
    }
  };

  onMounted(() => {
    window.addEventListener('resize', onResize, false);
    onResize();
  });

  onUnmounted(() => {
    window.removeEventListener('resize', onResize, false);
  });

  return { ...toRefs(state) };
};

const usei18nDate = language => {
  const vm = Vue.prototype;
  const lang = ref(language);

  watch(language, currentValue => {
    lang.value = currentValue;
  });

  const format = (date, dateFormat) => {
    return vm.$dayjs(date).locale(lang.value).format(dateFormat);
  };

  const timeAgo = (date, time) => {
    return vm.$dayjs(date).locale(lang.value).to(vm.$dayjs(time));
  };

  const getDate = unit => {
    return vm.$dayjs().get(unit);
  };

  const utc = (date, format) => {
    return vm.$dayjs(date).utc().locale(lang.value).format(format);
  };

  const timezone = (date, local, format) => {
    return vm.$dayjs(date).tz(local).locale(lang.value).format(format);
  };

  const difference = (datePri, dateSec, unit) => {
    const datePrimary = vm.$dayjs(datePri);
    const dateSecondary = vm.$dayjs(dateSec);

    return datePrimary.diff(dateSecondary, unit);
  };

  return {
    format,
    timeAgo,
    getDate,
    utc,
    timezone,
    difference,
  };
};

/**
 * This function update value when another value changes (data-binding).
 * @param {initial} - The initial is the state used during the initial render.
 * @returns darkMode, setDarkMode
 */

const useDarkMode = () => {
  const [darkMode, setDarkMode] = useState(null);

  const switchColor = mode => {
    const el = document.querySelector('body');
    if (mode) {
      localStorage.setItem('darkMode', true);
      el.classList.add('theme-dark');
    } else {
      localStorage.setItem('darkMode', false);
      el.classList.remove('theme-dark');
    }
  };
  const checkColor = () => {
    if (window.matchMedia && window.matchMedia('(prefers-color-scheme: dark)').matches && localStorage.getItem('darkMode') === null) {
      setDarkMode(true);
    }
  };

  onMounted(() => {
    if (localStorage.getItem('darkMode') == 'true') {
      const el = document.querySelector('body');
      el.classList.add('theme-dark');
      setDarkMode(true);
    } else {
      setDarkMode(false);
    }
    checkColor();
  });

  const changeColor = computed({
    get() {
      return darkMode.value;
    },
    set(value) {
      switchColor(value);
    },
  });

  return {
    darkMode: changeColor,
    setDarkMode,
  };
};

/**
 * @param {function} fn This callback fn that you want to execute after the Interval
 * @param ms ms time in milliseconds
 * @returns isActive, start, stop,
 */

const useInterval = (fn, ms = 100) => {
  let interval = null;
  const [active, setActive] = useState(false);

  const remove = () => {
    if (!interval) return;
    clearInterval(interval);
    interval = null;
  };

  const stop = () => {
    setActive(false);
    remove();
  };

  const start = () => (ms >= 0 ? setIntervalFn() : null);

  const setIntervalFn = () => {
    setActive(true);
    remove();
    interval = setInterval(fn, ms);
  };

  if (typeof window !== 'undefined') {
    start();
  }

  onMounted(stop);

  return {
    isActive: active,
    start,
    stop,
  };
};

/**
 * This function wait a certain amount of time before running again.
 * So built to limit the number of times a function is called.
 * @param delay Delay time in milliseconds
 * @returns debounceVal, value, debounceListener
 */

const useDebounce = (delay = 1000) => {
  let timeoutRef = null;
  const [value, setValue] = useState('');
  const [debounceVal, setDebounce] = useState('');

  const debounceListener = event => {
    if (timeoutRef !== null) {
      clearTimeout(timeoutRef);
    }

    setValue(event.target.value);
    timeoutRef = setTimeout(() => setDebounce(event.target.value), delay);
  };

  return { debounceVal, value, debounceListener };
};

const useStringCase = (lang = 'en') => {
  const locale = ref(lang);

  const camelCase = str =>
    str
      .replace(/(?:^\w|[A-Z]|\b\w)/g, function (word, index) {
        return index === 0 ? word.toLocaleLowerCase(locale) : word.toLocaleUpperCase(locale);
      })
      .replace(/\s+/g, '');

  const kebabCase = str =>
    str
      .match(/[A-Z]{2,}(?=[A-Z][a-z]+[0-9]*|\b)|[A-Z]?[a-z]+[0-9]*|[A-Z]|[0-9]+/g)
      .map(item => item.toLocaleLowerCase(locale))
      .join('-');

  const pascalCase = str =>
    str.replace(/\w\S*/g, word => word.charAt(0).toLocaleUpperCase(locale) + word.substr(1).toLocaleLowerCase(locale));

  const capitalizeCase = str => {
    const camelized = str.replace(/[-_](\w)/g, (_, c) => c.toLocaleUpperCase(locale));
    return camelized[0].toLocaleUpperCase(locale) + camelized.slice(1);
  };

  const lowerCase = str => str.toLocaleLowerCase(locale);

  const upperCase = str => str.toLocaleUpperCase(locale);

  const sentenceCase = str => str.toLocaleLowerCase(locale).replace(/(^\s*\w|[.!?]\s*\w)/g, s => s.toLocaleUpperCase(locale));

  return {
    camelCase,
    kebabCase,
    pascalCase,
    upperCase,
    lowerCase,
    sentenceCase,
    capitalizeCase,
  };
};

/**
 * This function forces a function to wait a certain amount of time before running again.
 * So the function is built to limit the number of times a function is called.
 * @param {object} params This object contains two parameters
 * @param params.delay Delay time in milliseconds
 * @param params.immediate If immediate is true, the function triggers immediately and
 * then waits for the interval before being called again.
 * @param {function} func This callback fn that you want to execute after the debounce time
 * @returns {function} debouncedFn
 */

const useDebounceFn = ({ delay, immediate }, func) => {
  let timer = null;
  let initial = false;
  const debouncedFn = (...args) => {
    const callNow = resolve => {
      resolve(func(...args));
    };

    if (!timer && immediate && !initial) {
      return new Promise(resolve => {
        initial = true;
        callNow(resolve);
      });
    }

    clearTimeout(timer);

    return new Promise(resolve => {
      timer = setTimeout(() => callNow(resolve), delay);
    });
  };

  return debouncedFn;
};

const useLocalStorage = (storageKey, defaultValue = '') => {
  const value = ref(null);
  const init = () => {
    const item = localStorage.getItem(storageKey);
    if (item !== null) {
      value.value = parseItem(item);
      return;
    }

    value.value = defaultValue;
  };

  const parseItem = item => {
    let value = null;
    try {
      value = JSON.parse(item);
    } catch {
      value = item;
    }

    return value;
  };

  const handler = event => {
    if (event.key === storageKey) {
      value.value = event.newValue ? parseItem(event.newValue) : null;
    }
  };

  let initialized = false;

  if (typeof window !== 'undefined') {
    init();
    initialized = true;
  }

  onMounted(() => {
    if (!initialized) {
      init();
    }

    window.addEventListener('storage', handler, true);
  });

  watchEffect(() => {
    if (value.value) {
      localStorage.setItem(storageKey, JSON.stringify(value.value));
    }
  });

  onUnmounted(() => {
    window.removeEventListener('storage', handler);
  });

  const remove = () => {
    try {
      localStorage.removeItem(storageKey);
      value.value = null;
    } catch (error) {
      // localStorage can throw.
      console.error(error);
    }
  };

  return {
    value,
    remove,
  };
};

const stringifyObject = (object, brace = ';') => {
  const keyValuePairs = Object.entries(object).filter(pair => pair[0] || pair[1]);

  return keyValuePairs.reduce((acc, item, index) => {
    if (index === keyValuePairs.length - 1) {
      return (acc = acc + `${item[0]}=${item[1]}`);
    }

    return (acc = acc + `${item[0]}=${item[1]}${brace}`);
  }, '');
};

const useCookie = (cookieArg = document.cookie) => {
  const cookie = ref(cookieArg);

  const parseCookie = () => {
    return cookie.value.split(';').reduce((object, cookieString) => {
      const splitCookie = cookieString.trim().split('=');
      try {
        return { ...object, [splitCookie[0]]: JSON.parse(splitCookie[1]) };
      } catch (error) {
        return { ...object, [splitCookie[0]]: splitCookie[1] };
      }
    }, {});
  };

  const setCookie = (key, value, options = {}) => {
    const optionsString = stringifyObject(options);

    cookie.value = `${cookie.value};${key}=${value}${optionsString}`;
  };

  const appendCookie = (key, value, specialCharacter = ',') => {
    const parsedCookie = parseCookie();
    parsedCookie[key] = `${parsedCookie[key]}${specialCharacter}${value}`;

    cookie.value = stringifyObject(parsedCookie);
  };

  const getCookie = key => {
    const cookieObject = parseCookie();

    return cookieObject[key];
  };

  const deleteCookie = key => {
    // expiration date is set to an undoubtedly past date for to be deleted immediately
    const parsedCookie = parseCookie();
    parsedCookie[key] = ';expires=Thu Jan 01 1970 00:00:00 GMT"';

    cookie.value = stringifyObject(parsedCookie);
  };

  return {
    cookie,
    setCookie,
    appendCookie,
    parseCookie,
    getCookie,
    deleteCookie,
  };
};

const useClipboard = () => {
  const copy = (value, container = document) => {
    try {
      const tempInput = document.createElement('textarea');
      tempInput.value = value;
      tempInput.setAttribute('readonly', '');
      container.appendChild(tempInput);
      tempInput.select();

      document.execCommand('copy');
      container.removeChild(tempInput);
    } catch (error) {
      // eslint-disable-next-line no-console
      console.error(error);
    }
  };

  return {
    copy,
  };
};

export { useBind, useClipboard, useCookie, useDarkMode, useDate, useDebounce, useDebounceFn, useEmbed, useInterval, useList, useLocalStorage, useModal, usePick, useQueue, useResize, useState, useStringCase, usei18nDate };
